from __future__ import print_function
from traceback import format_exception
import datetime
from .format_time import format_time


class Manager:
    def __init__(self, handle_exceptions=True, verbose=False, precision=0):
        self.handle_exceptions = handle_exceptions
        self.verbose = verbose
        self.precision = precision
        self.result = None

    def __enter__(self):
        self.start_time = datetime.datetime.now()
        return self

    def __exit__(self, type, value, traceback):
        duration = int(
            round(
                (datetime.datetime.now() - self.start_time).total_seconds(),
                self.precision,
            )
        )

        if type in [KeyboardInterrupt, SystemExit]:
            raise value

        if value:
            formatted = format_exception(type, value, traceback)
            self.serialized_result = {
                "success": False,
                "result": None,
                "error": formatted,
                "duration_seconds": duration,
            }
            if self.handle_exceptions:
                print(
                    "Error handled by werkit (to disable, pass `handle_exceptions=False` to werkit.manage_execution)"
                )
                print(traceback)
                return True
            else:
                return False
        else:
            self.serialized_result = {
                "success": True,
                "result": self.result,
                "error": None,
                "duration_seconds": duration,
            }

        if self.verbose:
            print("Completed in {}".format(format_time(duration)))



from setuptools import setup, find_packages

# Set version_info[__version__], while avoiding importing numpy, in case numpy
# and vg are being installed concurrently.
# https://packaging.python.org/guides/single-sourcing-package-version/
version_info = {}
exec(open("werkit/package_version.py").read(), version_info)

with open("README.md") as f:
    readme = f.read()

with open("requirements.txt") as f:
    install_requires = f.read()

setup(
    name="werkit",
    version=version_info["__version__"],
    description="Toolkit for encapsulating Python-based computation into deployable and distributable tasks",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Metabolize",
    author_email="github@paulmelnikow.com",
    url="https://github.com/metabolize/werkit",
    project_urls={
        "Issue Tracker": "https://github.com/metabolize/werkit/issues",
        "Documentation": "https://werkit.readthedocs.io/en/stable/",
    },
    packages=find_packages(),
    install_requires=install_requires,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: System :: Distributed Computing",
        "Topic :: Scientific/Engineering",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
    ],
)



import os
import pytest
from redis import Redis
from rq import Worker
from .invoke import (
    DEFAULT_QUEUE_NAME,
    clean,
    invoke_for_each,
    get_results,
    get_aggregate_status,
)
from .testing import square, multiply


def path_to_redis():
    import distutils.spawn

    return distutils.spawn.find_executable("redis-server")


if os.environ.get("CI"):

    @pytest.fixture
    def dummy_fixture():
        pass

    @pytest.fixture
    def conn_fixture():
        return Redis()

    new_redis_proc = dummy_fixture
    redis_conn = conn_fixture
else:
    from pytest_redis.factories import redis_proc, redisdb

    new_redis_proc = redis_proc(executable=path_to_redis(), logsdir="/tmp")
    redis_conn = redisdb("new_redis_proc")


def test_invoke_for_each(new_redis_proc, redis_conn):
    items = {"item_{}".format(i): i for i in range(10)}
    invoke_for_each(square, items, connection=redis_conn)
    assert get_aggregate_status(connection=redis_conn) == "queued"

    Worker([DEFAULT_QUEUE_NAME], connection=redis_conn).work(burst=True)

    assert get_aggregate_status(connection=redis_conn) == "finished"

    assert get_results(connection=redis_conn) == {k: x * x for k, x in items.items()}


def test_repeat_invoke_raises_error(new_redis_proc, redis_conn):
    items = {"item_{}".format(i): i for i in range(10)}

    clean(connection=redis_conn)

    invoke_for_each(square, items, connection=redis_conn)
    with pytest.raises(ValueError):
        invoke_for_each(square, items, connection=redis_conn)


def test_invoke_for_each_with_kwargs(new_redis_proc, redis_conn):
    items = {"item_{}".format(i): i for i in range(10)}
    invoke_for_each(
        multiply, items, kwargs={"multiplier": 5}, clean=True, connection=redis_conn
    )

    Worker([DEFAULT_QUEUE_NAME], connection=redis_conn).work(burst=True)

    assert get_results(connection=redis_conn) == {k: 5 * x for k, x in items.items()}
